/* 
***********************************************************

  Spectool: Tool to reduce the resolution of spectra
            by first smooting to a fixed resolution and
            subsequent re-binning to a new wavelength grid.

  written by: Ralf Kotulla
  (c) 2008-2009, The GALEV Team

  compile via: g++ -lgsl spectool.cpp -o spectool

  Requires: GNU Scientific library (GSL)

  How to use:
    spectool (file with lambdas) (file with fluxes) (resolution in A)
  Output:
    File spec.out, with 1st col: lambdas,
                        2nd col: input spectrum,
                        3rd col: smoothed spectrum
                        
***********************************************************
*/

#include <iostream>
#include <fstream>
#include <string>
#include <cmath>
#include <iomanip>
#include <sstream>
#include <vector>
#include <stdio.h>
#include <stdlib.h>
#include <gsl/gsl_spline.h>

using namespace std;

#define PI acos(-1)

int main(int argc, char* argv[])
{
    cout << endl
         << "  ConvolveQE:                   (c) 2009, GALEV Team" << endl
         << "  Convolve filter response  curve with  the  quantum" << endl
         << "  efficiency of the dectectors to enable computation" << endl
         << "  of accurate  magnitudes by  GALEV / COLORS / COCOS" << endl
         << endl;
    
    if (argc <= 3) {
        cout << "  Usage:" << endl
             << "  ./ConvolveQE filter.in detector.QE filter.out" << endl
             << endl;
        return -1;
    }
    
        
    string filterfile(argv[1]);
    string qefile(argv[2]);
    string outfile(argv[3]);
    string line;

    vector<double> fc_lambda;
    vector<double> fc_response;
    double fc_peak = 0;
    
    vector<double> qe_lambda;
    vector<double> qe_response;
    double qe_peak = 0;
    
    double dum_lambda, dum_response;
    
    
    ifstream f_fc(filterfile.c_str(), ios_base::in);
    if (!f_fc.good()) {
        cout << "  Unable to open filter filter " << filterfile << endl
             << endl;
        return -1;
    }
    while (getline(f_fc, line, '\n')) {
        istringstream iss(line);
        if (line.length() <= 0) continue;
        if (line.at(0) == '#') continue;
        iss >> dum_lambda >> dum_response;
        fc_lambda.push_back(dum_lambda);
        fc_response.push_back(dum_response);
        if (dum_response > fc_peak) fc_peak = dum_response;
    }
    f_fc.close();
    if (fc_lambda.size() <= 1) {
        cout << "  Filter-file is too short or invalid" << endl
             << endl;
        return -1;
    } else {
        cout << "  Read " << fc_lambda.size() << " entries from filter "
             << filterfile << endl;
    }
    
   
    
    ifstream f_qe(qefile.c_str(), ios_base::in);
    if (!f_qe.good()) {
        cout << "  Unable to open filter filter " << qefile << endl
             << endl;
        return -1;
    }
    while (getline(f_qe, line, '\n')) {
        istringstream iss(line);
        if (line.length() <= 0) continue;
        if (line.at(0) == '#') continue;
        iss >> dum_lambda >> dum_response;
        qe_lambda.push_back(dum_lambda);
        qe_response.push_back(dum_response);
        if (dum_response > qe_peak) qe_peak = dum_response;
    }
    f_qe.close();
    if (qe_lambda.size() <= 1) {
        cout << "  Detector QE-file is too short or invalid" << endl
             << endl;
        return -1;
    } else {
        cout << "  Read " << qe_lambda.size() << " entries from QE file "
             << qefile << endl;
    }


    cout << "  Performing convolution ..." << flush;
    
    gsl_interp_accel *acc = gsl_interp_accel_alloc();

    // For Spline interpolation, use the following line
    // gsl_interp *spline = gsl_interp_alloc(gsl_interp_cspline, qe_lambda.size());

    // For linear interpolation use this line:
    gsl_interp *interp = gsl_interp_alloc(gsl_interp_linear, qe_lambda.size());

    gsl_interp_init(interp, &qe_lambda.at(0), &qe_response.at(0),
                    qe_lambda.size());

    double qe_res, total_res;
    ofstream out(outfile.c_str(), ios_base::out);
    for (int i=0; i<fc_lambda.size(); i++) {
        qe_res = gsl_interp_eval(interp,
                                 &qe_lambda.at(0), &qe_response.at(0),
                                 fc_lambda.at(i), acc);

        total_res = fc_response.at(i) * qe_res / (qe_peak * fc_peak);
        out << fc_lambda.at(i) << " " << total_res << endl;
    }
    out.close();
    gsl_interp_free(interp);
    gsl_interp_accel_free(acc);
    
    cout << " done!" << endl;
    
    cout << endl;
} // END OF PROGRAM

